#!/etc/opt/openit/perl

###############################################################################
#
# 
# 


BEGIN {
    push @INC, "/opt/openit/contrib/lib";
}

use strict;
use if $^O eq 'MSWin32' , "OpeniT::Config";
use OpeniT::Contrib::Config;
use Getopt::Long;

my $opt_verbose;
my $opt_debug; 
my $opt_dryrun;
my $opt_help;

unless ( GetOptions("verbose"		=> \ $opt_verbose,
					"debug"		  => \ $opt_debug,
					"help"		   => \ $opt_help)) {
	&print_usage;
}


my $config = new OpeniT::Contrib::Config;
if( $^O eq 'MSWin32' ) {
	$config->read(&OpeniT::Config::CONFIG_DIR . "/openit.cfg");    # Additional config
} else {
	$config->read("/etc/opt/openit/openit_contrib.cfg");    # Additional config
}

my $config_dir = $config->{CONFIG_DIR};
$config_dir =~ tr{\\}{/};
if (! -d $config_dir) {
	die "Unable to locate Open iT config ($config_dir) directory";
}

#Get argument
my $num_args = $#ARGV + 1;
my $filter_to_find;

if( $num_args > 0 ) {
	$filter_to_find = $ARGV[0];
}

my $map_file = "${config_dir}/package-mapping";
my $filtrator_dir = "filtrator";

if($^O eq 'MSWin32' ) { 
	$filtrator_dir = "Components/filtrator";
}

# flexlog2 only needs init and event filters
my @filter_files = &get_filters_to_update("$config_dir/$filtrator_dir", "init-flex|events-flex");
# # get all licpoll2 and samcollect 
push @filter_files, &get_filters_to_update("$config_dir/$filtrator_dir", "license-", "flex-log");
# # freeze filters
push @filter_files, &get_filters_to_update("$config_dir/$filtrator_dir", "freeze-");
# # ASEE filters
push @filter_files, &get_filters_to_update("$config_dir/$filtrator_dir", "olap", "winapp");

print "Updating filters\n";

foreach my $filter (@filter_files) {
	if ($^O eq 'MSWin32' ){
		&update_filter_xml( $filter );
	} else {
		&update_filter( $filter );
	}
}

&create_mapping_file($map_file);

exit 0;



sub get_filters_to_update {
	my ($filter_dir, $include, $exclude) = @_;

	opendir(DIR,$filter_dir);
	my @tmpfilters = readdir(DIR);
	closedir(DIR);
	
	if ($^O eq 'MSWin32' ){
		@tmpfilters = grep { /\.xml$/ } @tmpfilters;
	} else {
		@tmpfilters = grep { /\.conf$/ } @tmpfilters;
	}
	
	@tmpfilters = grep { /$include/ } @tmpfilters;

	if (defined($exclude) && $exclude ne "") {
		@tmpfilters = grep { !/$exclude/ } @tmpfilters;
	}
	$exclude = "old";
	@tmpfilters = grep { !/$exclude/ } @tmpfilters;
	
	my @filters;
	foreach my $file (@tmpfilters) {
        	push @filters, "$filter_dir/$file \n";
	}
	return @filters;
}


sub update_filter_xml {
	my ($filter) = @_;
	
	chomp $filter;
	$filter =~ s/^\s+|\s+$//g;
	

    open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);

	if ( grep {/Duplicate:packages/} @lines ) {	
		VERBOSE ("Filter '$filter' already updated." );
		return;
    } else {
		my @found = grep {/$filter_to_find/} @lines;
		if ( @found < 2 )
		{
			print "Cant find the filter on where to put package-mapping.";
			return;
		}
		print "Updating '$filter'...\n";
		&make_old_files( $filter );
	}

	my $ind = 0;
    $ind++ while( $lines[$ind] !~ /$filter_to_find/ );
    $ind += 3;

	my @insert = ("\t\t\t\t\t\<Object\>\n", "\t\t\t\t\t\t\<Name\>Duplicate\:packages\<\/Name\>\n");

	push @insert, "\t\t\t\t\t\t\t\<Description\>Duplicating packages, according to map file.\<\/Description\>\n\t\t\t\t\t\t\t\t\<SubObjects\>\n";
    splice @lines, $ind, 0, @insert;

    $ind += 3; # Skip inserted lines
	$ind++ while( $lines[$ind] !~ /filter-defs/ );
	splice @lines, $ind-1, 0, "\t\t\<\/SubObjects\>\n\t\t\t\<\/Object\>\n";
    $ind += 4; # Compensate for inserted line and then into object

	my @insert = ("\t\t\t\t\t\<Object\>\n",
		"\t\t\t\t\t\t\<Name\>Duplicate\:packages\<\/Name\>\n",
	       "\t\t\t\t\t\t\<Description\>Duplicate package\<\/Description\>\n",
	       "\t\t\t\t\t\t\<SubObjects\>\n",
	       "\t\t\t\t\t\t\t\<Object\>\n",
	       "\t\t\t\t\t\t\t\t\<Name\>inactive\<\/Name\>\n",
	       "\t\t\t\t\t\t\t\t\<Description\>Switch to turn the feature renaming to on\<\/Description\>\n",
	       "\t\t\t\t\t\t\t\t\<Value type\=\"Bool\">false\<\/Value\>\n",
	       "\t\t\t\t\t\t\t\<\/Object\>\n",
	       "\t\t\t\t\t\t\t\<Object\>\n",
	       "\t\t\t\t\t\t\t\t\<Name\>filter\<\/Name\>\n",
	       "\t\t\t\t\t\t\t\t\<Description\>Feature rename Filter\<\/Description\>\n",
	       "\t\t\t\t\t\t\t\t\<Value type\=\"String\">Duplicate\<\/Value\>\n",
	       "\t\t\t\t\t\t\t\<\/Object\>\n",
	       "\t\t\t\t\t\t\t\<Object\>\n",
	       "\t\t\t\t\t\t\t\t\<Name\>map-file\<\/Name\>\n",
	       "\t\t\t\t\t\t\t\t\<Description\>map file location\<\/Description\>\n",
	       "\t\t\t\t\t\t\t\t\<Value type\=\"FileName\"\>\$\{OpeniT\.directories\.conf\}package\-mapping\<\/Value\>\n",
	       "\t\t\t\t\t\t\t\<\/Object\>\n",
	       "\t\t\t\t\t\t\t\<Object\>\n",
	       "\t\t\t\t\t\t\t\t\<Name\>source\<\/Name\>\n",
	       "\t\t\t\t\t\t\t\t\<Description\>Source\<\/Description\>\n",
	       "\t\t\t\t\t\t\t\t\<Value type\=\"String\">package\<\/Value\>\n",
	       "\t\t\t\t\t\t\t\<\/Object\>\n",
	       "\t\t\t\t\t\t\t\<Object\>\n",
	       "\t\t\t\t\t\t\t\t\<Name\>target\<\/Name\>\n",
	       "\t\t\t\t\t\t\t\t\<Description\>Target\<\/Description\>\n",
	       "\t\t\t\t\t\t\t\t\<Value type\=\"String\">package\<\/Value\>\n",
	       "\t\t\t\t\t\t\t\<\/Object\>\n",
		"\t\t\t\t\t\t\<\/SubObjects\>\n",
	       "\t\t\t\t\t\<\/Object\>\n" );
	splice @lines, $ind, 0, @insert;

	open(OUT, ">$filter") || die "Unable to overwrite filter file '$filter'";
	print OUT @lines;
	close(OUT);
}


sub update_filter {
	my ($filter) = @_;
	
	# trim white space
	chomp $filter;
	$filter =~ s/^\s+|\s+$//g;
	
    VERBOSE( "Updating conf file '$filter'" );
	
  	my @lines = _read_file( $filter );
	if ( grep {/Duplicate:packages/} @lines ) {
		VERBOSE( "Filter '$filter' already updated." );
		return;
    } 
	
my @duplicate_package_def = (
"		Duplicate:packages()=\"\"\n",
"		{\n",
"			inactive(Bool)=\"false\"{}\n",
"			filter(String)=\"Duplicate\"{}\n",
"			limited()=\"\"\n",
"			{\n",
"				map-file(FileName)=\"\${OpeniT.directories.conf}package-mapping\"{}\n",
"				source(String)=\"package\"{}\n",
"				target(String)=\"package\"{}\n",
"			}\n",
"		}\n" );


my @calcsum_def = ( 
"		CalcSum:package()=\"\"\n",
"		{\n",
"			inactive(Bool)=\"false\"{}\n",
"			filter(String)=\"CalcSum\"{}\n",
"			compare(String)=\"product,package,feature,feature_version,origsource\"\n",
"			{\n",
"				field(String)=\"signals\"{}\n",
"				positive(Bool)=\"true\"{}\n",
"				pattern(String)=\"1\"{}\n",
"			}\n",
"			remove(String)=\"origsource\"{}\n",
"			summarize(String)=\"licenses\"\n",
"			{\n",
"				field(String)=\"signals\"{}\n",
"				positive(Bool)=\"true\"{}\n",
"				pattern(String)=\"1\"{}\n",
"			}\n",
"		}\n");

	

	my $find = "CalcSum:feature";
	

	 	
	if ( ! grep {/CalcSum:*/} @lines ) {
		$find = "Duplicate:features";
	}
	
	#for freeze-actions
	if ( grep {/freeze-actions.*/} @lines ) {
		$find = "Duplicate:products";
	} 
	
	&_add_filter_process( \@lines, \@duplicate_package_def , $find , "Duplicate:packages" , 5 );

	if ( grep {/CalcSum:*/} @lines ) {
		&_add_filter_process( \@lines, \@calcsum_def , "Duplicate:packages" , "CalcSum:package" , 6 );
    } 
	
	_save_filter( $filter ,@lines );
}

sub make_old_files {
	my ($filter) = @_;
	print "Creating $filter backup\n";
	my $tmpfname = basename($filter);
	my $tmpdirname = dirname($filter);
	my @splitted_fname = split('\.', $tmpfname);
	my $extension = $splitted_fname[1];
	my $fname = $splitted_fname[0];
	my $old = "$tmpdirname/$fname-old\.$extension";

    open(FILE, $filter) || die "Unable to read filter file '$filter'";
    my @lines = <FILE>;
    close(FILE);
	
	open(OUT, ">$old") || die "Unable to create backup file '$old'";
    print OUT @lines;
    close(OUT);
}


sub create_mapping_file {
	my ($map) = @_;

	if(-e $map) {
		print "Mapping file already present\n";
		return;
    }

    print "Adding mapping file '$map'\n";

    open(OUT, ">$map") || die "Unable to open map file '$map' for writing";
    print OUT "

#### Package Mapping ####
#########################

MAP: ELM-VxWorks-Edition-Combined
FL_SE_GPP_VE*
FL_SE_PXX_VE*

MAP: ELM-ADD-OCD-Combined
FL_SE_ADD_OCD_4WB*

MAP: ELM-VxWorks-MILS-Combined
FL_SE_WB_VX_MILS*

";
	close(OUT);
}

sub _add_filter_process
{
	my ( $lines, $insert_def, $find, $name ,$tabs ) = @_;
	
	if ( ! grep {/$name/} @$lines ) 
	{
		if ( ! grep {/$find/} @$lines )
		{
			DEBUG( "Unable to find '$find'" );
			return;
		}
		DEBUG ( "Adding Filter process '$name' locating '$find'" );
		my $ind = 0;

		$ind++ while(@$lines[$ind] !~ /filter-process/ );
		$ind++ while(@$lines[$ind] !~ /$find/ );
		
		my $tab;
		for (my $i=0; $i <= $tabs; $i++) {
		  $tab .= "\t";
		}
		$ind+= 2;
		splice @$lines , $ind, 0 , "$tab$name()=\"\"\n" ;
		$ind++;
		splice @$lines , $ind , 0 , "$tab\{\n" ;
		
		$ind++;
		
		# fix tabbings
		while(@$lines[$ind] !~ /filter-defs/ )
		{
			@$lines[$ind] = "\t" . @$lines[$ind];
			$ind++;
		}
		
		splice @$lines , $ind , 0 , "\t\}\n" ;
		
		&_add_filter_def ( \@$lines, \@$insert_def , $name );
	}
	else
	{
		DEBUG ( "Filter process '$name' already exist" );
	}
}

sub _add_filter_def
{
	my ( $lines, $insert_def, $name ) = @_;
	my $ind = 0;
	$ind++ while( @$lines[$ind] !~ /filter-defs/ );
	splice @$lines , $ind + 2 , 0 , @$insert_def ;
}

sub _read_file
{
	my ($cfg) = @_;
	open(IN, "$cfg") || die "Error: Unable to read cfg '$cfg'!\n";
	my @lines = <IN>;
	close(IN);
	return @lines;
}

sub DEBUG
{
	if ( defined $opt_debug )
	{
		my ($message) = @_;
		chomp $message;
		print "DEBUG: " . $message . "\n";
	}
}

sub VERBOSE
{
	if ( defined $opt_verbose )
	{
		my ($message) = @_;
		chomp $message;
		print " -- " . $message . "\n";
	}
}
sub _save_filter
{
	my ( $cfg, @lines ) = @_; 
	
	my $old = "$cfg.old";
	my $new = "$cfg.new";
	
	open(OUT, ">$new") || die "Error: Can't write new cfg '$new'!\n";
	print OUT @lines;
	close(OUT);

	if( ! rename($cfg, $old)) {
		die "Error: Unable to previous config to '$old'!\n";
	}

	if( ! rename($new, $cfg)) {
		die "Error: Unable to rename new config to '$cfg'!\n";
	}
}


